@@ -23,13 +23,13 @@ use Carp;
 use FileHandle;
 use Cwd qw( abs_path );
 use UI::Dialog::Backend;
-use File::Slurp::Unicode;
+use File::Slurp;
 use String::ShellQuote;
 
 BEGIN {
   use vars qw( $VERSION @ISA );
   @ISA = qw( UI::Dialog::Backend );
-  $VERSION = '1.15';
+  $VERSION = '1.16';
 }
 
 #::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::::
